package models.file;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

/**
 * Represents a node in the tree structure of a file system of files 
 * and directories. A node can either be a parent directory or a single
 * file.
 * 
 * @author NorthPoint
 *
 */
public class FileTreeNode {

	public File file;
    public FileTreeNode parent;
    public List<FileTreeNode> children;
    
    /**
     * Default constructor
     */
    private FileTreeNode() {
    	children = new ArrayList<FileTreeNode>();
    }
    
    /**
     * Full constructor
     * @param file
     * @param parent
     */
    public FileTreeNode(File file, FileTreeNode parent) {
    	this();
    	this.file = file;
    	this.parent = parent;
    }
    
    /**
     * Add a child node to the parent node.
     * @param node
     */
    public void addChild(FileTreeNode node) {
    	children.add(node);
    }
    
    /**
     * Returns true if the node is a directory or false if the node is a file.
     * @return Whether the node is a directory
     */
    public boolean isDirectory() {
    	return file != null && !file.isFile();
    }
    
    /**
     * Returns true if the node has children or false if the node 
     * has no child elements.
     * @return Whether the node has children
     */
    public boolean hasChildren() {
    	return !children.isEmpty();
    }
}
