package models.util.file;

import java.io.File;
import java.io.FileNotFoundException;

/**
 * Utility class for File operations.
 * 
 * @author NorthPoint
 */
public class FileUtil {

	/* All supported image file extensions */ 
	public static String[] IMAGE_FILE_EXTS = new String[] {"png", "gif"}; //jpeg is not supported
	static {
		for(int i=0; i<IMAGE_FILE_EXTS.length; i++) {
			IMAGE_FILE_EXTS[i] = IMAGE_FILE_EXTS[i].toLowerCase();
		}
	}
	
	/**
	 * Validate a file directory and throw appropriate IOExceptions where an error
	 * is detected.
	 * 
	 * @param dir
	 * @throws FileNotFoundException
	 */
	public static void validateDirectory (File dir) throws FileNotFoundException {
		if (dir == null) throw new IllegalArgumentException("Directory should not be null.");
		if (!dir.exists()) throw new FileNotFoundException("Directory does not exist: " + dir);
		if (!dir.isDirectory()) throw new IllegalArgumentException("Is not a directory: " + dir);
		if (!dir.canRead()) throw new IllegalArgumentException("Directory cannot be read: " + dir);
	}
}
