package models.util.image;

import java.io.File;
import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;

import play.Logger;

import models.file.ImageDirectory;

public class DataPie {
	List<DataSlice> slices = new LinkedList<DataSlice>();

	private int height;
	private int width;
	
	ArrayList<Integer> data;
	
	private DataPie() {
		
	}
	
	public static DataPie init() {
		return new DataPie();
	}

	public static DataPie init(DataSlice[] s) {
		DataPie answer = new DataPie();
		answer.addSlices(s);
		return answer;
	}
	
	public void fuseData() {
		setDimensions();
		
		for(DataSlice slice:slices) {
			data.addAll(slice.getPaddedData(width));
		}	
	}
	
	//sets width to the max width of DataSlices
	private void setDimensions() {
		int maxWidth = 0;
		int totalHeight = 0;
		
		for(DataSlice slice:slices) {
			maxWidth = Math.max(maxWidth, slice.getWidth());
			slice.setVerticalPos(totalHeight);
			totalHeight += slice.getHeight();
		}
		
		//invert vertical position to count from bottom as CSS does
		for(DataSlice slice:slices) {
			slice.setVerticalPos(totalHeight - slice.getVerticalPos());
		}

		width = maxWidth;
		height = totalHeight;
		data = new ArrayList<Integer>(width * height);
	}
	
	public int getHeight() {
		if(height < 1) fuseData();
		return height;
	}

	public int getWidth() {
		if(width < 1) fuseData();
		return width;
	}

	public void addSlice(DataSlice slice) {
		slices.add(slice);
		reset();
	}

	public void addSlices(DataSlice[] s) {
		for(DataSlice slice:s) {
			addSlice(slice);
		}
		reset();
	}

	public void addSlices(Collection<DataSlice> s) {
		slices.addAll(s);
		reset();
	}

	public List<DataSlice> getSlices() {
		if(data == null) {
			fuseData();
		}
		return Collections.unmodifiableList(slices);
	}

	public ArrayList<Integer> getData() {
		if(data == null) {
			fuseData();
		}
		
		return data;
	}

	public int[] getIntData() {
		ArrayList<Integer> d = getData();
		
		int[] answer = new int[d.size()];
		int index = 0;
		for(Integer i:d) {
			answer[index] = i.intValue();
			index++;
		}
		return answer;
		
	}

	private void reset() {
		data = null;
		height = 0;
		width = 0;
	}
}
