package models.util.image;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.apache.sanselan.ImageInfo;

import play.Logger;
import util.ConfigReader;

/**
 *
 * @author NorthPoint Solutions
 */
public class DataSlice {

	public ImageInfo imageInfo;
	private List<Integer> data;
	private int width;
	private int height;
	private String fileName;
	private String path;
	
	private int horizontalPos;
	private int verticalPos;

	private DataSlice() {
		data = new ArrayList<Integer>(1000);
	}

	public static DataSlice init(String filename, String path, int[] pixels, int height, int width) {
		DataSlice answer = new DataSlice();
		answer.setFileName(filename);
		answer.setPath(path);
        answer.setData(pixels);
        answer.setHeight(height);
        answer.setWidth(width);
        return answer;
	}
	
	public List<Integer> getData() {
		return data;
	}
	
	public List<Integer> getPaddedData(int desiredWidth) {
		if(desiredWidth < width) throw new IllegalArgumentException("when padding img data, desiredWidth must be >= current width");		
		else if(desiredWidth == width) return data;
	
		ArrayList<Integer> answer = new ArrayList<Integer>(Collections.nCopies(desiredWidth * height, 0));

		for(int h=0;h<height;h++) {
			for(int w=0; w<width; w++) {
				int oldIndex = h * width + w;
				int newIndex = h * desiredWidth + w;
				answer.set(newIndex, data.get(oldIndex));
			}
		}
		return answer;
	}
	
	public int[] getIntData() {
		int[] answer = new int[data.size()];
		int index = 0;
		for(Integer i:data) {
			answer[index] = i.intValue();
			index++;
		}
		return answer;
	}


	public void setData(int[] rawdata) {
		data = new ArrayList<Integer>(rawdata.length);
		for(int i:rawdata) {
			data.add(i);
		}
	}

	public void addDatum(Integer datum) {
		addDatum(datum);
	}

	public int getWidth() {
		return width;
	}

	public void setWidth(int width) {
		this.width = width;
	}

	public int getHeight() {
		return height;
	}

	public void setHeight(int height) {
		this.height = height;
	}

	public int getHorizontalPos() {
		return horizontalPos;
	}

	public void setHorizontalPos(int horizontalPos) {
		this.horizontalPos = horizontalPos;
	}

	public int getVerticalPos() {
		return verticalPos;
	}

	public void setVerticalPos(int verticalPos) {
		this.verticalPos = verticalPos;
	}


	public String getFileName() {
		return fileName;
	}

	public String getFileNameSansExt() {
		if(fileName == null) return null;
		
		return fileName.replaceFirst("\\..*$", "");
	}

	public void setFileName(String name) {
		this.fileName = name;
	}
	
	public String getPath() {
		return path;
	}
	
	public void setPath(String path) {
		this.path = path;
	}

	/*
	private static String replaceLast(String string, String toReplace, String replacement) {
	    int pos = string.lastIndexOf(toReplace);
	    if (pos > -1) {
	        return string.substring(0, pos)
	             + replacement
	             + string.substring(pos + toReplace.length(), string.length());
	    } else {
	        return string;
	    }
	}
	 */
	
	public String getCssDirPath() {
		String cssDirPath;
		if (ConfigReader.cssDirPathSeperator.value() != null) {
			cssDirPath = path.replaceAll("/", ConfigReader.cssDirPathSeperator.value());
		} else {
			cssDirPath = "";
		}
		return cssDirPath;
	}

	public List<Integer> getRow(int rownum) {
		List<Integer> row = data.subList(
		    rownum * getWidth(),
		    Math.min( (rownum+1) * getWidth(), data.size())
		);		
		return row;
	}

	@Override
	public String toString() {
		StringBuilder answer = new StringBuilder();
		answer.append("height: ").append(getHeight()).append(", ");
		answer.append("width: ").append(getWidth()).append(", ");
		answer.append("data size: ").append(data.size()).append(", ");
		answer.append("h x w = size?: ").append(getHeight() * getWidth() == data.size());
		return answer.toString();
	}

	@Override
	public boolean equals(Object obj) {
		if(obj == null) return false;
		if(! (obj instanceof DataSlice)) return false;

		DataSlice p = (DataSlice)obj;
		return data.equals(p.data);
	}
}
