package models.util.image;

import java.awt.image.BufferedImage;
import java.awt.image.PixelGrabber;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;

import org.apache.sanselan.ImageFormat;
import org.apache.sanselan.ImageWriteException;
import org.apache.sanselan.Sanselan;

import exception.SpeedySpriteException;

import play.Logger;
import util.ConfigReader;

/**
 * alternate name ideas: spritepack, spriteglue
 */
public class FusionMgr {
	
	public final static String 	OUTPUT_FILE_EXT = "png";
	
	/*
	 * writes the fused image to the outputstream specified
	 */
	public DataPie fuse(Collection<File> images, OutputStream os) {
		DataPie dp = DataPie.init();
		
		for(File f:images) {
			DataSlice ds = asDataSlice(f.getName(), f.getPath(), loadImage(f));
			dp.addSlice(ds);
		}
		
		if(os != null)
			save(dp, os);

		return dp;
	}
	
	/*
	 * uses a dynamic hash filename to write the fused image in a standard temp dir
	 */
	public DataPie fuse(List<File> images, File outputFile) {
		FileOutputStream fos = null;
		DataPie answer = null;
    	
    	Logger.debug("Ouput File: %s", outputFile.getPath());
		
		try {
			if(outputFile != null)
				fos = new FileOutputStream(outputFile);
			answer = fuse(images, fos);
		} catch(Exception e) {
			e.printStackTrace();
		} finally {
			if(fos != null) {
				try {
					fos.close();
				} catch(Exception e) {
					e.printStackTrace();
				}
			}
		}
		return answer;
	}
	
	public static String hashName(List<File> files) {
		List<String> filepaths = new ArrayList<String>(files.size());
		for (File file : files) {
			filepaths.add(file.getPath());
		}
		Collections.sort(filepaths);
		StringBuilder preHash = new StringBuilder();
		for (String path : filepaths) {
			preHash.append(path);
		}
		return hashName(preHash.toString());
	}

	
    private BufferedImage loadImage(File ref) {
        BufferedImage bimg = null;
        try {
            bimg = Sanselan.getBufferedImage(ref);
        } catch (Exception e) {
        	Logger.debug("Unable to load image file: " + e);
        	throw new SpeedySpriteException("Unable to load file: " + ref + 
        			" Please ensure file is a valid image file of a supported type.", e);
        }
        return bimg;
    }

    private DataSlice asDataSlice(String name, String path, BufferedImage image) {
        PixelGrabber grabber;
        int width = image.getWidth();
        int height = image.getHeight();

        int[] pixels = new int[width * height];

        try {
            grabber = new PixelGrabber(image, 0, 0, width, height,pixels,0,width);
            grabber.grabPixels(0);
        } catch (Exception e) {
            e.printStackTrace(); // TODO handle error
        }

        DataSlice answer = DataSlice.init(name, path, pixels, height, width);
        return answer;
    }

    public static File generatedHashFile(List<File> images) {
    	return new File(generatedHashFilePath(images));
    }

    public static  String generatedHashFilePath(List<File> images) {
		String fileHash = hashName(images);
		String filename = ConfigReader.spriteCacheDir + fileHash;
		return filename;
    }

    private void save(DataPie payload, OutputStream os) {
        BufferedImage bi = new BufferedImage(payload.getWidth(), payload.getHeight(), BufferedImage.TYPE_INT_ARGB);
        Logger.debug(payload.toString());
        bi.setRGB(0, 0, payload.getWidth(), payload.getHeight(), payload.getIntData(), 0, payload.getWidth());
        save(bi, os);
    }
    
    private void save(BufferedImage img, OutputStream os) {
        try {
            Sanselan.writeImage(img, os, ImageFormat.IMAGE_FORMAT_PNG, null);
        } catch (IOException e) {
            e.printStackTrace();
        } catch(ImageWriteException iwe) {
            iwe.printStackTrace();
        }
    }
    
    public static String hashName(String s) {
    	MessageDigest md = null;
    	try {
	        md = MessageDigest.getInstance("MD5");
	        md.update(s.getBytes());
    	} catch(NoSuchAlgorithmException nsa) {
    		nsa.printStackTrace();
    	}
    	
        byte byteData[] = md.digest();
 
        //convert the byte to hex format method 1
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < byteData.length; i++) {
        	sb.append(Integer.toString((byteData[i] & 0xff) + 0x100, 16).substring(1));
        }
        sb.append("." + OUTPUT_FILE_EXT);
        return sb.toString();
    }
}
